# Read in information
fin = open("s2sis.txt", 'r')
t = [ row.split(' ') for row in fin.read().split('\n') if row ]
n, m = int(t[0][0]), int(t[0][1])

# Create graph in an adjacency list form
adj_list = [ [] for i in range(n) ]
for line in t[1:]:
    a, b, u = int(line[0])-1, int(line[1])-1, int(line[2])
    # We'll convert input to 0-based indexes
    # 0-based is nicer to work with
    #
    # Let's create a directed graph.
    # If there's voltage u between a and b, then
    # we create an edge with weight -u from a to b
    # and an edge with weight u from b to a
    adj_list[a].append((b,-u))
    adj_list[b].append((a,u))

# We keep track of how many nodes we've already reached
# because it might consist of several components, not necessarily all connected
reached = 0
# We keep track of "absolute" electric potentials ("voltages with respect to ground")
voltages = [None]*n
# To speed up looking up which components we have not yet considered
# we keep track of the minimum index where we have already started a component from
fst = 0

contradiction = False # We start out assuming there is no contradiction
while reached < n: # While we have not yet visited every node:
    # We have to find a connected component we have not yet explored
    # We start looking from the last connected component's first node we found
    while voltages[fst] != None:
        fst += 1
    
    # We can start out by assuming that this node has voltage 0 with respect to ground
    # If it is something else, it won't matter, because everything in this component
    # will simply be shifted and any contradictions that exist still do and
    # no additional contradictions will be made
    voltages[fst] = 0
    stack = adj_list[fst][:] # We put the neighbours of this first voltage on the stack
    reached += 1
    
    # Perform DFS on the graph
    while stack != []:
        nextnode, nextvoltage = stack[-1] # Grab what we have on the stack currently
        del(stack[-1]) # pop the stack
        if voltages[nextnode] != None: # Check if we already have info on what potential it should be
            # If our already existing info on what the potential should be
            # contradicts the voltage we get from the stack, then we have a contradiction
            if voltages[nextnode] != nextvoltage:
                contradiction = True
                break
        else:
            # If there is no voltage with respect to ground for this node yet,
            # we set it to what it should be
            voltages[nextnode] = nextvoltage
            reached += 1
            # Add all neighbours of this node to the stack along with info on
            # what their voltages with respect to ground should be,
            # calculated by adding the voltage with respect to ground of current node
            # plus voltage between current node and the node being pushed on stack
            stack += [ (edge[0], edge[1] + nextvoltage) for edge in adj_list[nextnode] ]
    
    if contradiction:
        break

# Write output to file
fout = open("s2val.txt", 'w')
if contradiction:
    fout.write("EI\n")
else:
    fout.write("JAH\n")
